;********************************************************
;
;             Gear Shifter Control
;
;                                 Author : Fabian Josef
;********************************************************

        list            P=16F84A
		include			P16F84A.INC

		__CONFIG	_RC_OSC & _WDT_OFF & _PWRTE_ON & _CP_OFF


;****************  Label Definition  ********************
u_btn   equ     h'00'           ;RA0 bit - Up Shift Button
d_btn   equ     h'01'           ;RA1 bit - Down Shift Button
c_btn   equ     h'02'           ;RA2 bit - Auto Up Shift Switch

cnt500u equ     h'0c'           ;500usec counter Address
cnt1m   equ     h'0d'           ;1msec counter Address
cnt100m equ     h'0e'           ;100msec counter Address
cnt500m equ     h'0f'           ;500msec counter Address
cnt1s   equ     h'10'           ;1sec counter Address

cl_ext  equ     0   ; was b'00000001'     ;Extend clutch (1 = extend, 0 = release)
cl_vlv  equ     1   ; was b'00000010'     ;Clutch release valve (1 = launch, 0 = drive)
cl_led  equ     2   ; was b'00000100'     ;Clutch in LED (1 = on, 0 = off)
gr_ext  equ     3   ; was b'00001000'     ;Extend gear (1 = extend, 0 = release)
gr_dir  equ     4   ; was b'00010000'     ;Gear direction (1 = up, 0 = down)
ig_cut  equ     5   ; was b'00100000'     ;Ignition cut (1 = on, 0 = off)

;****************  Initiate Process  *********************
init    org		h'00'
		movlw 	b'00000000'
		movwf 	PORTB			;Set all PORTB pins = 0
		bsf     STATUS,RP0      ;Change to Bank1 
        movlw   h'ff'           ;Set input mode data
        movwf   TRISA           ;Set PORTA to Input mode
		movlw   h'00'           ;Set output mode data
        movwf   TRISB           ;Set PORTB to Output mode
        bcf     STATUS,RP0      ;Change to Bank0
        call	cl_hold    		;Disengage Clutch

;**************  Button Press Process  ******************
btn		btfss   PORTA,u_btn     ;Up-Button Pressed?
        call    ch_up           ;Yes. Call Change Up
        btfss   PORTA,d_btn     ;Down-Button Pressed?
        call    ch_down         ;Yes. Call Change Down
        btfss   PORTA,c_btn     ;Neutral-Button Pressed?
        call    cl_hold         ;Yes. Call Neutral
        goto    btn		        ;Retry

;***********  Change Up Subroutine ***************
ch_up   bsf		PORTB,gr_dir	;Set gear direction up
		bsf		PORTB,ig_cut	;Set ignition cut on
		call	t100m			;Delay 100ms
		bsf		PORTB,gr_ext	;Set gear extend on
		call	t100m			;Delay 100ms
		bcf		PORTB,ig_cut	;Set ignition cut off
		call	t100m			;Delay 100ms
		bcf		PORTB,gr_ext	;Set gear extend off
        return

;***********  Change Down Subroutine ***************
ch_down bcf		PORTB,gr_dir	;Set gear direction down
		bcf		PORTB,cl_vlv	;Set clutch release valve to drive
		bsf		PORTB,cl_ext	;Set clutch extend on
		call	t100m			;Delay 100ms
		bsf		PORTB,gr_ext	;Set gear extend on
		call	t500m			;Delay 500ms
		bcf		PORTB,gr_ext	;Set gear extend off
		call	t100m			;Delay
        btfss   PORTA,d_btn     ;Down-Button Pressed?
		goto	$-1				;Yes? Wait until released
		bcf		PORTB,cl_ext	;Set clutch extend off
		return	

;***********  Clutch Hold Subroutine ***************
cl_hold bsf		PORTB,cl_ext	;Set clutch extend on
        call    t100m           ;Wait 100msec
        return

;********************************************************
;          Timer  Subroutine for 10MHz clock
;********************************************************

;*************  1msec Timer Subroutine  *****************
t1m     movlw   d'2'            ;(1)       Set loop cnt1
        movwf   cnt1m           ;(1)       Save loop cnt1
tm1lp1  movlw   d'249'          ;(1)*2     Set loop cnt2
        movwf   cnt500u         ;(1)*2     Save loop cnt2
tm1lp2  nop                     ;(1)*249*2 Time adjust
        nop                     ;(1)*249*2 Time adjust
        decfsz  cnt500u,F       ;(1)*249*2 cnt500u-1=0 ?
        goto    tm1lp2          ;(2)*248*2 No, continue
        decfsz  cnt1m,F         ;(1)*2     cnt1m-1=0 ?
        goto    tm1lp1          ;(2)       No. Continue
        return                  ;(2)       Yes. Cnt end
                                ;Total 2501*0.4usec=1msec

;*************  100msec Timer Subroutine  ***************
t100m   movlw   d'100'          ;Set loop counter
        movwf   cnt100m         ;Save loop counter
tm2lp   call    t1m             ;1msec subroutine
        decfsz  cnt100m,f       ;cnt100m - 1 = 0 ?
        goto    tm2lp           ;No. Continue
        return                  ;Yes. Count end


;*************  500msec Timer Subroutine  ***************
t500m   movlw   d'5'            ;Set loop counter
        movwf   cnt500m         ;Save loop counter
tm3lp   call    t100m           ;100msec subroutine
        decfsz  cnt500m,f       ;cnt500m - 1 = 0 ?
        goto    tm3lp           ;No. Continue
        return                  ;Yes. Count end

        end